##############################################################################
#
# Copyright (c) 2008-2009 SIA "KN dati". (http://kndati.lv) All Rights Reserved.
#                    General contacts <info@kndati.lv>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields,osv
import tools
from mx.DateTime import RelativeDateTime, now

class trans_driver_categories(osv.osv):
	_name = "trans.driver_categories"
	_rec_name = "lic_catg"

	_columns = {
		'lic_catg': fields.char('Licence Categories', size=4, required=True),
		'description': fields.char('Description', size=64),
	}

trans_driver_categories()

class trans_make(osv.osv):
	_name = "trans.make"

	_columns = {
		'name': fields.char('Brandname', size=64, required=True),
		'model': fields.one2many('trans.model', 'make_id', 'Models'),
	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['name']):
			name = str(r['name'])
			res.append((r['id'], name))
		return res

trans_make()

class trans_model(osv.osv):
	_name = "trans.model"

	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'make_id': fields.many2one('trans.make', 'Make'),
	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['name']):
			name = str(r['name'])
			res.append((r['id'], name))
		return res

trans_model()

class trans_fuel(osv.osv):
	_name = "trans.fuel"

	_columns = {
		'name': fields.char('Name', size=20, required=True, translate=True),
	}

trans_fuel()

def _get_licence_catg(self, cr, uid, context={}):
	obj = self.pool.get('trans.driver_categories')
	ids = obj.search(cr, uid, [])
	res = obj.read(cr, uid, ids, ['lic_catg'], context)
	return [(r['lic_catg'], r['lic_catg']) for r in res]

class trans_vehicle(osv.osv):
	_name = "trans.vehicle"
	_rec_name = "model"

	def search(self, cr, uid, args, offset=0, limit=None, order=None,
			context=None, count=False):
		if context is None:
			context = {}

		if 'driver_id' in context.keys():
			catg = []
			obj = self.pool.get('trans.driver_categories')
			driver = self.pool.get('trans.driver')
			catg_ids = map(int, driver.browse(cr, uid, context['driver_id'], {}).lic_catg)
			for c in obj.browse(cr, uid, catg_ids, {}):
				catg.append(c.lic_catg)
			args = [['driving_licence_catg','in',catg]]

		return super(trans_vehicle,self).search(cr, uid, args, offset, limit,
				order, context=context, count=count)

	def _get_reg_data(self, cr, uid, ids, type):
		res={}
		reg = self.pool.get('trans.reg')
		for p in self.browse(cr, uid, ids, {}):
			date = '01-01-1900'
			if p.registration==[]:
				res[p.id] = False
				continue
			reg_ids = map(int, p.registration)
			field = False
			for id in reg_ids:
				r = reg.browse(cr, uid, id, {})
				if type == 'lic_plate' and r.lic_plate: field = r.lic_plate
				elif type == 'holder' and r.act_holder: field = r.act_holder
				elif type == 'holder' and r.holder: field = r.holder
				if not field:
					res[p.id] = False
				elif r.reg_date >= date:
					if type == 'lic_plate': res[p.id] = field
					elif type == 'holder': res[p.id] = field.id
					date = r.reg_date
		return res

	def _get_lic_plate(self, cr, uid, ids, field, arg=None, context={}):
		return self._get_reg_data(cr, uid, ids, 'lic_plate')

	def _get_curr_holder(self, cr, uid, ids, field, arg=None, context={}):
		return self._get_reg_data(cr, uid, ids, 'holder')

	_columns = {
		'make': fields.many2one('trans.make', 'Make', required=True),
		'model': fields.many2one('trans.model', 'Model', domain="[('make_id', '=', make)]", required=True),
		'chassis_number': fields.char('Chassis number', size=64),
		'colour': fields.char('Colour', size=64),
		'fuel': fields.many2many('trans.fuel', 'trans_vehic_fuel_rel', 'vehicle_id', 'fuel_id', 'Fuel type(s)'),
		'manf_year': fields.integer('Year of manufacture'),
		'eng_volume': fields.integer('Engine volume'),
		'full_mass': fields.integer('Full mass'),
		'self_mass': fields.integer('Self mass'),
		'seats': fields.integer('Seats'),
		'standing': fields.integer('Standing'),
		'notes': fields.text('Notes'),
		'registration': fields.one2many('trans.reg', 'vehicle_id', 'Registration'),
		'lic_plate': fields.function(_get_lic_plate, method=True, string='Licence plate', type='char', size=128),
		'curr_holder': fields.function(_get_curr_holder, method=True, string='Current Holder', type='many2one', obj="res.partner"),
		'holder': fields.many2one('res.partner', 'Holder'),
		'driving_licence_catg': fields.selection(_get_licence_catg, 'Licence Categories'),
		'active': fields.boolean('Active'),
	}

	def change_registration(self, cr, uid, ids, field):
		data = {}
		date = '01-01-1900'
		if not field:
			return {'value':{}}
		reg = self.pool.get('trans.reg')
		for r in field:
			holder_id = r[2]['holder']
			if holder_id and r[2]['reg_date'] >= date:
				data['curr_holder'] = holder_id
				date = r[2]['reg_date']
		return {'value':data}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['model','make','lic_plate']):
			name = str(r['make'][1]) + ' ' + str(r['model'][1])
			if r['lic_plate']:
				 name += ', ' + str(r['lic_plate'])
			res.append((r['id'], name))
		return res

	def _get_curr_holder_constr(self, cr, uid, ids):
		result = False
		reg = self.pool.get('trans.reg')
		for p in self.browse(cr, uid, ids, {}):
			reg_ids = map(int, p.registration)
			for id in reg_ids:
				r = reg.browse(cr, uid, id, {})
				if r.holder: result = r.holder.id
		if result:cr.execute("""
					UPDATE trans_vehicle
					SET holder=%s
					WHERE id in (%s)
				   """,(result, ','.join(map(str,ids))))
		return True

	_defaults = {
		'manf_year' : lambda *a: int(now().strftime('%Y')),
		'active' : lambda *a: True,
	}

	_constraints = [
        	(_get_curr_holder_constr, '', ['holder']),
    	]

trans_vehicle()

class trans_driver(osv.osv):
	_name = "trans.driver"

	_columns = {
		'transporter': fields.many2one('res.partner', 'Transporter', required=True),
		'name': fields.many2one('res.partner.address', 'Name', domain="[('partner_id', '=', transporter)]", required=True),
		'default_vehicle': fields.many2one('trans.vehicle', 'Default vehicle'),
		'lic_catg': fields.many2many('trans.driver_categories', 'trans_driver_licence_rel', 'driver_id', 'catg_id', 'Driving Licence Categories'),

	}

	def name_get(self, cr, user, ids, context={}):
		if not len(ids):
			return []
		res = []
		for r in self.read(cr, user, ids, ['name']):
			name = str(r['name'][1])
			res.append((r['id'], name))
		return res

	def clear_name(self, cr, uid, ids, field):
		if not field:
			return {'value':{}}
		return {'value':{'name':False}}

	def _check_default_vehicle(self, cr, uid, ids):
		catg_obj = self.pool.get('trans.driver_categories')
		for p in self.browse(cr, uid, ids, {}):
			for id in map(int, p.lic_catg):
				if p.default_vehicle.driving_licence_catg != catg_obj.browse(cr, uid, id, {}).lic_catg:
					raise osv.except_osv('Driving licence!', 'Selected driving licence do not permit drive selected vehicle.')
					return False
		return True

	_constraints = [
        	(_check_default_vehicle, '', ['default_vehicle']),
    	]

trans_driver()

class trans_reg(osv.osv):
	_name = "trans.reg"
	_rec_name = "number"

	_columns = {
		'number': fields.char('Certificate ID', size=64, required=True),
		'lic_plate': fields.char('Licence plate', size=64),
		'reg_date': fields.date('Registration date', required=True),
		'effective_date': fields.date('Effective date'),
		'authority': fields.many2one('res.partner', 'Authority'),
		'authority_address': fields.many2one('res.partner.address', 'Authority\'s address', domain="[('partner_id', '=', authority)]"),
		'owner': fields.many2one('res.partner', 'Owner'),
		'owner_address': fields.many2one('res.partner.address', 'Owner\'s address', domain="[('partner_id', '=', owner)]"),
		'holder': fields.many2one('res.partner', 'Holder'),
		'holder_address': fields.many2one('res.partner.address', 'Holder\'s address', domain="[('partner_id', '=', holder)]"),
		'act_holder': fields.many2one('res.partner', 'Actual Holder'),
		'act_holder_address': fields.many2one('res.partner.address', 'Actual Holder\'s address', domain="[('partner_id', '=', act_holder)]"),
		'vehicle_id': fields.many2one('trans.vehicle', 'Vehicle'),
	}

	def clear_address(self, cr, uid, ids, field):
		if not field:
			return {'value':{}}
		elif field == 'authority':
			return {'value':{'authority_address':False}}
		elif field == 'owner':
			return {'value':{'owner_address':False}}
		elif field == 'holder':
			return {'value':{'holder_address':False}}
		elif field == 'act_holder':
			return {'value':{'act_holder_address':False}}

trans_reg()

