##############################################################################
#
# Copyright (c) 2008-2009 SIA "KN dati". (http://kndati.lv) All Rights Reserved.
#                    General contacts <info@kndati.lv>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import fields,osv
import tools
import netsvc
import pooler
from mx import DateTime
from mx.DateTime import now
import time

class trans_request_config(osv.osv):
	_name = "trans.request_config"

	_columns = {
		'req_users': fields.many2many('hr.timesheet.group', 'trans_request_rel', 'config_id', 'tgroup_id', 'Work Team'),
	}

trans_request_config()

class trans_insurance_type(osv.osv):
	_name = "trans.insurance.type"

	_columns = {
		'name': fields.char('Name', size=64, required=True),
		'code': fields.char('Code', size=32, required=True),
		'required': fields.boolean('Required'),
	}

	_defaults = {
		'required' : lambda *a: True,
	}

trans_insurance_type()

class trans_insurance_required(osv.osv):
	_name = "trans.insurance.required"

	_columns = {
		'name': fields.char('Policy Number', size=32, required=True, translate=True),
		'code': fields.char('Code', size=32, required=True),
	}

trans_insurance_required()

def _get_insurance_type(self, cr, uid, context={}):
	obj = self.pool.get('trans.insurance.type')
	ids = obj.search(cr, uid, [])
	res = obj.read(cr, uid, ids, ['name', 'code'], context)
	return [(r['code'], r['name']) for r in res]

def _get_insurance_required(self, cr, uid, context={}):
	obj = self.pool.get('trans.insurance.required')
	ids = obj.search(cr, uid, [])
	res = obj.read(cr, uid, ids, ['name', 'code'], context)
	return [(r['code'], r['name']) for r in res]

def find_req_users(self, cr, uid):
	result = []
	admin_id = self.pool.get('res.users').search(cr, uid, [('login','=','admin')])[0]
	config_obj = self.pool.get('trans.request_config')
	ids = config_obj.search(cr, uid, [])
	if ids:
		config_id = ids[0]
		config = config_obj.browse(cr, uid, config_id, {})
		tgroup = map(int, config.req_users)
		tg_obj = self.pool.get('hr.timesheet.group')
		for wt in tgroup:
			user = tg_obj.browse(cr, uid, wt, {}).manager.id
			if user not in result:
				result.append(tg_obj.browse(cr, uid, wt, {}).manager.id)
		emp_obj = self.pool.get('hr.employee')
		emp_ids = emp_obj.search(cr, uid, [])
		for id in emp_ids:
			emp_rec = emp_obj.browse(cr, uid, id, {})
			for r in map(int, config.req_users):
				if r in map(int, emp_rec.workgroups):
					user = emp_rec.user_id.id
					if user not in result:
						result.append(emp_rec.user_id.id)
	if result and result[0]:
		return result
	else:
		return [admin_id]

def create_request(self, cr, uid, subject, req_text):
	req_users = find_req_users(self, cr, uid)
	for user_id in req_users:
		admin_id = self.pool.get('res.users').search(cr, uid, [('login','=','admin')])[0]
		req_values = {'name':subject,'act_from':admin_id,'act_to':user_id,'date_sent':time.strftime('%Y-%m-%d %H:%M:%S'),'body':req_text,'state':'waiting','priority':'2'}
		id = self.pool.get('res.request').create(cr, uid, req_values)
		cr.execute('select act_from,act_to,body,date_sent from res_request where id=%d', (id,))
		values = cr.dictfetchone()
		if len(values['body']) > 128:
			values['name'] = values['body'][:125] + '...'
		else:
			values['name'] = values['body'] or '/'
		values['req_id'] = id
		self.pool.get('res.request.history').create(cr, uid, values)
	return True

class trans_insurance(osv.osv):
	_name = "trans.insurance"

	def _check_insurance(self, cr, uid, id, request_date, context={}):
		insurance = self.browse(cr, uid, id, {})
		current_date = now()
		if current_date.strftime('%Y-%m-%d') >= request_date:
			vehicle_obj = self.pool.get('trans.vehicle')
			vehicle_name = vehicle_obj.name_get(cr, uid, [insurance.vehicle_id.id])[0][1]
			vehicle_insurances =  map(int, insurance.vehicle_id.insurance_ids)
			cr.execute("""SELECT id, time_effective_end, time_effective_start
				      FROM trans_insurance
				      WHERE active=true and id in (%s)
				   """ % (','.join(map(str,vehicle_insurances))))
			insurance_dates = cr.fetchall()
			insr_OK = False
			for insur_id, end, start in insurance_dates:
				if insur_id!=id and end>=current_date.strftime('%Y-%m-%d %H:%M:%S') and start>=current_date.strftime('%Y-%m-%d %H:%M:%S'):
					insr_OK = True
					break

			if not insr_OK:
				insr_type_obj = self.pool.get('trans.insurance.type')
				type_id = insr_type_obj.search(cr, uid, [('code','=',insurance.insurance_type)], limit=1)
				insr_name = insr_type_obj.browse(cr, uid, type_id[0], {}).name
				if insurance.required == 'default':
					insr_req = insr_type_obj.browse(cr, uid, type_id[0], {}).required
				elif insurance.required == 'required':
					insr_req = True
				else: insr_req = False

				if insr_req:
					days_left = (DateTime.strptime(insurance.time_effective_end,'%Y-%m-%d %H:%M:%S')-current_date).day+1
					req_body = 'Insurance on the '+vehicle_name+' will expire soon after '+\
						str(days_left)+' days.\n\n'+'End Date: '+insurance.time_effective_end[:10]+\
						'\nPolicy Number: '+insurance.name+'\nInsurance Type: '+insr_name+'\n'
					create_request(self, cr, uid, '['+vehicle_obj.name_get(cr, uid, [insurance.vehicle_id.id])[0][1]+'] Insurance will expire soon!', req_body)
		if current_date.strftime('%Y-%m-%d') >= insurance.time_effective_end[:10]:
			self.write(cr, uid, [id], {'active':False})
			self.pool.get('ir.cron').unlink(cr, uid, [insurance.cron_id.id])
		return True

	def _get_days_left(self, cr, uid, ids, field, arg=None, context={}):
		res = {}
		for p in self.browse(cr, uid, ids, context):
			res[p.id] = (DateTime.strptime(p.time_effective_end, '%Y-%m-%d %H:%M:%S') - now()).day+1
			if res[p.id]<0: res[p.id]=0
		return res

	_columns = {
		'name': fields.char('Policy Number', size=64, required=True),
		'vehicle_id': fields.many2one('trans.vehicle', 'Vehicle'),
		'date_issued': fields.datetime('Date Issued'),
		'time_effective_start': fields.datetime('Effective From', required=True),
		'time_effective_end': fields.datetime('Effective till', required=True),
		'insurance_type': fields.selection(_get_insurance_type, 'Insurance Type', required=True),
		'insurance_company': fields.many2one('res.partner', 'Insurance Company', required=True),
		'insurant_id': fields.many2one('res.partner.address', 'Insurant'),
		'risk_level': fields.char('Risk level', size=32),
		'active': fields.boolean('Active'),
		'self_risk': fields.boolean('Risk for Own Account'),
		'lead_time': fields.integer('Procurement Time'),
		'warn_manager': fields.boolean('Warn manager'),
		'cron_id': fields.many2one('ir.cron', 'Scheduler'),
		'required': fields.selection(_get_insurance_required, 'Required'),
		'invoice_id': fields.many2one('account.invoice', 'Invoice'),
		'invoice_line_id': fields.many2one('account.invoice.line', 'Invoice Line'),
		'insurance_premium': fields.float('Insurance Premium'),
		'days_left': fields.function(_get_days_left, method=True, string='Days left', type='integer'),
	}

	_defaults = {
		'lead_time' : lambda *a: 5,
		'required' : lambda *a: 'default',
		'warn_manager' : lambda *a: True,
		'active' : lambda *a: True,
	}

	def create(self, cr, uid, vals, context={}):
		if not context:
			context={}
		vals=vals.copy()
		c = context.copy()
		c['novalidate'] = True
		id = super(trans_insurance, self).create(cr, uid, vals, c)
		if vals['active'] and vals['warn_manager']:
			insr_type_obj = self.pool.get('trans.insurance.type')
			type_id = insr_type_obj.search(cr, uid, [('code','=',vals['insurance_type'])], limit=1)
			insr_name = insr_type_obj.browse(cr, uid, type_id[0], {}).name
			res = {'name':insr_name+' - '+vals['name'], 'model':'trans.insurance', 'args':[id], 'function':'_check_insurance', 'user_id':uid,\
				'interval_type':'days','interval_number':1,'doall':True}
			res['numbercall'] = (DateTime.strptime(vals['time_effective_end'],'%Y-%m-%d %H:%M:%S') - now()).day+1
			res['args'].append((DateTime.strptime(vals['time_effective_end'],'%Y-%m-%d %H:%M:%S') - DateTime.RelativeDateTime(days=vals['lead_time']+3)).strftime('%Y-%m-%d'))
			#res['nextcall'] = (DateTime.strptime(vals['time_effective_end'],'%Y-%m-%d %H:%M:%S') - DateTime.RelativeDateTime(days=vals['lead_time']+3)).strftime('%Y-%m-%d %H:%M:%S')
			cron_id = self.pool.get('ir.cron').create(cr, uid, res)
			self.write(cr, uid, [id], {'cron_id':cron_id})
		return id

	def write(self, cr, uid, ids, vals, context=None):
		if not context:
			context={}
		vals=vals.copy()
		r = self.browse(cr, uid, ids[0], {})
		if 'active' not in vals: vals['active'] = r.active
		if 'warn_manager' not in vals: vals['warn_manager'] = r.warn_manager
		if vals['active'] and vals['warn_manager']:
			insr_type_obj = self.pool.get('trans.insurance.type')
			if 'insurance_type' in vals:
				insurance_type = vals['insurance_type']
			else:
				insurance_type = r.insurance_type
			type_id = insr_type_obj.search(cr, uid, [('code','=',insurance_type)], limit=1)
			insr_name = insr_type_obj.browse(cr, uid, type_id[0], {}).name
			if 'name' in vals:
				name = vals['name']
			else:
				name = r.name
			res = {'name':insr_name+' - '+name, 'model':'trans.insurance', 'args':[r.id], 'function':'_check_insurance', 'user_id':uid,\
				'interval_type':'days','interval_number':1, 'doall':True}
			if 'time_effective_end' in vals:
				time_effective_end = vals['time_effective_end']
			else:
				time_effective_end = r.time_effective_end
			if 'lead_time' in vals:
				lead_time = vals['lead_time']
			else:
				lead_time = r.lead_time
			res['numbercall'] = (DateTime.strptime(time_effective_end,'%Y-%m-%d %H:%M:%S') - now()).day+1
			res['args'].append((DateTime.strptime(time_effective_end,'%Y-%m-%d %H:%M:%S') - DateTime.RelativeDateTime(days=lead_time+3)).strftime('%Y-%m-%d'))
			#res['nextcall'] = (DateTime.strptime(time_effective_end,'%Y-%m-%d %H:%M:%S') - DateTime.RelativeDateTime(days=lead_time+3)).strftime('%Y-%m-%d %H:%M:%S')
			if vals['active'] and not r.cron_id and 'cron_id' not in vals:
				vals['cron_id'] = self.pool.get('ir.cron').create(cr, uid, res)
			else:
				self.pool.get('ir.cron').write(cr, uid, r.cron_id.id, res)
		elif not vals['active'] or not vals['warn_manager'] and r.cron_id: self.pool.get('ir.cron').unlink(cr, uid, [r.cron_id.id])
		return super(trans_insurance, self).write(cr, uid, ids, vals, context=context)

	def unlink(self, cr, uid, ids, context={}):
		for cron_id in self.read(cr, uid, ids, ['cron_id']):
			if cron_id['cron_id']: self.pool.get('ir.cron').unlink(cr, uid, [cron_id['cron_id'][0]])
		osv.osv.unlink(self, cr, uid, ids, context)
		return True

trans_insurance()

class trans_vehicle(osv.osv):
	_name = "trans.vehicle"
	_inherit = "trans.vehicle"

	_columns = {
		'required_insurance_types': fields.many2many('trans.insurance.type', 'trans_required_insurance_rel','vehicle_id',\
			'insurance_type_id', 'Required Insurances'),
		'insurance_ids': fields.one2many('trans.insurance', 'vehicle_id', 'Insurances'),
	}

trans_vehicle()

